/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('UserProfile', function () {
  var users = void 0;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('user-profile', function () {
      done();
    });
  });

  beforeEach(function () {
    users = { charmstore: {
        user: 'user-dalek',
        usernameDisplay: 'test owner'
      } };
  });

  afterEach(function () {
    window.flags = {};
  });

  it('renders a populated user profile page', function () {
    var acl = {};
    var links = [];
    var addNotification = sinon.stub();
    var changeState = sinon.stub();
    var getDiagramURL = sinon.stub();
    var listBudgets = sinon.stub();
    var listModelsWithInfo = sinon.stub();
    var destroyModels = sinon.stub();
    var switchModel = sinon.stub();
    var getAgreements = sinon.stub();
    var staticURL = 'test-url';
    var user = users.charmstore;
    var charmstore = {};
    window.flags = { blues: true };
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      acl: acl,
      addNotification: addNotification,
      users: users,
      charmstore: charmstore,
      destroyModels: destroyModels,
      getAgreements: getAgreements,
      getDiagramURL: getDiagramURL,
      listBudgets: listBudgets,
      listModelsWithInfo: listModelsWithInfo,
      switchModel: switchModel,
      interactiveLogin: true,
      changeState: changeState,
      pluralize: sinon.stub(),
      staticURL: staticURL,
      storeUser: sinon.stub(),
      user: user }), true);
    var instance = component.getMountedInstance();
    var output = component.getRenderOutput();
    var content = output.props.children.props.children;
    var emptyComponent = React.createElement(juju.components.EmptyUserProfile, {
      staticURL: staticURL,
      switchModel: switchModel });
    var lists = [React.createElement(juju.components.UserProfileModelList, {
      acl: acl,
      addNotification: addNotification,
      ref: 'modelList',
      key: 'modelList',
      currentModel: undefined,
      destroyModels: destroyModels,
      listModelsWithInfo: listModelsWithInfo,
      switchModel: switchModel,
      user: user }), React.createElement(juju.components.UserProfileEntityList, {
      ref: 'bundleList',
      key: 'bundleList',
      changeState: changeState,
      charmstore: charmstore,
      getDiagramURL: getDiagramURL,
      type: 'bundle',
      user: user,
      users: users }), React.createElement(juju.components.UserProfileEntityList, {
      ref: 'charmList',
      key: 'charmList',
      changeState: changeState,
      charmstore: charmstore,
      getDiagramURL: getDiagramURL,
      type: 'charm',
      user: user,
      users: users }), React.createElement(juju.components.UserProfileAgreementList, {
      ref: 'agreementList',
      key: 'agreementList',
      getAgreements: getAgreements,
      user: user }), React.createElement(juju.components.UserProfileBudgetList, {
      ref: 'budgetList',
      key: 'budgetList',
      listBudgets: listBudgets,
      user: user })];
    var expected = React.createElement(
      'div',
      { className: 'inner-wrapper' },
      React.createElement(juju.components.UserProfileHeader, {
        users: users,
        avatar: '',
        interactiveLogin: instance._interactiveLogin,
        links: links,
        username: users.charmstore.usernameDisplay }),
      React.createElement(
        'div',
        null,
        React.createElement(
          juju.components.SectionLoadWatcher,
          {
            EmptyComponent: emptyComponent,
            timeout: 10 },
          lists
        )
      )
    );
    assert.deepEqual(content, expected);
  });

  it('does not pass the charmstore login if interactiveLogin is falsey', function () {
    var pluralize = sinon.stub();
    pluralize.withArgs('model', sinon.match.any).returns('models');
    var links = [];
    var output = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      charmstore: {},
      switchModel: sinon.stub(),
      listBudgets: sinon.stub(),
      listModelsWithInfo: sinon.stub(),
      changeState: sinon.stub(),
      getAgreements: sinon.stub(),
      getDiagramURL: sinon.stub(),
      interactiveLogin: false,
      pluralize: pluralize,
      storeUser: sinon.stub(),
      user: users.charmstore }));
    var expected = React.createElement(juju.components.UserProfileHeader, {
      users: users,
      avatar: '',
      interactiveLogin: undefined,
      links: links,
      username: users.charmstore.usernameDisplay });
    assert.deepEqual(output.props.children.props.children.props.children[0], expected);
  });

  it('can log in to charmstore fetch macaroons from the bakery', function () {
    var macaroon = sinon.spy();
    var storeUser = sinon.stub();
    var charmstore = {
      bakery: {
        fetchMacaroonFromStaticPath: sinon.stub().callsArgWith(0, null, macaroon)
      }
    };
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      switchModel: sinon.stub(),
      users: users,
      listBudgets: sinon.stub(),
      listModelsWithInfo: sinon.stub(),
      changeState: sinon.stub(),
      charmstore: charmstore,
      getAgreements: sinon.stub(),
      getDiagramURL: sinon.stub(),
      interactiveLogin: true,
      pluralize: sinon.stub(),
      storeUser: storeUser,
      user: users.charmstore }), true);
    var instance = renderer.getMountedInstance();
    instance._interactiveLogin();
    assert.equal(charmstore.bakery.fetchMacaroonFromStaticPath.callCount, 1);
    assert.equal(storeUser.callCount, 1);
  });

  it('skips the budget list when blues flag is inactive', function () {
    var addNotification = sinon.stub();
    var changeState = sinon.stub();
    var getDiagramURL = sinon.stub();
    var listBudgets = sinon.stub();
    var listModelsWithInfo = sinon.stub();
    var switchModel = sinon.stub();
    var getAgreements = sinon.stub();
    var user = users.charmstore;
    window.flags = { blues: false };
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      addNotification: addNotification,
      users: users,
      charmstore: {},
      getAgreements: getAgreements,
      getDiagramURL: getDiagramURL,
      listBudgets: listBudgets,
      listModelsWithInfo: listModelsWithInfo,
      switchModel: switchModel,
      interactiveLogin: true,
      changeState: changeState,
      pluralize: sinon.stub(),
      storeUser: sinon.stub(),
      user: user }), true);
    var instance = component.getMountedInstance();
    assert.isUndefined(instance.refs.budgetList);
  });

  it('skips the bundle and charm lists when not logged in', function () {
    var changeState = sinon.stub();
    var getDiagramURL = sinon.stub();
    var listBudgets = sinon.stub();
    var listModelsWithInfo = sinon.stub();
    var switchModel = sinon.stub();
    var getAgreements = sinon.stub();
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: {},
      charmstore: {},
      getAgreements: getAgreements,
      getDiagramURL: getDiagramURL,
      listBudgets: listBudgets,
      listModelsWithInfo: listModelsWithInfo,
      switchModel: switchModel,
      interactiveLogin: true,
      changeState: changeState,
      pluralize: sinon.stub(),
      storeUser: sinon.stub(),
      user: undefined }), true);
    var instance = component.getMountedInstance();
    assert.isUndefined(instance.refs.bundleList);
    assert.isUndefined(instance.refs.charmList);
  });
});